# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: radiensserver.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from . import common_pb2 as common__pb2
from . import datasource_pb2 as datasource__pb2
from . import biointerface_pb2 as biointerface__pb2
from . import spikesorter_pb2 as spikesorter__pb2
from . import radiens_dev_pb2 as radiens__dev__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x13radiensserver.proto\x12\x06\x61llego\x1a\x0c\x63ommon.proto\x1a\x10\x64\x61tasource.proto\x1a\x12\x62iointerface.proto\x1a\x11spikesorter.proto\x1a\x11radiens_dev.proto\"K\n\x19RadiensHealthcheckRequest\x12\x1a\n\x12isIncludeWarehouse\x18\x01 \x01(\x08\x12\x12\n\nisDetailed\x18\x02 \x01(\x08\"]\n\x16RadiensHealthcheckSpec\x12\x0e\n\x06\x65rrMsg\x18\x01 \x01(\t\x12\x33\n\toutfitter\x18\x02 \x01(\x0b\x32 .allego.WarehouseHealthcheckSpec\"\x1e\n\x0fRadiensClientId\x12\x0b\n\x03pid\x18\x01 \x02(\x05\"\x91\x01\n\x12SummaSessionsReply\x12?\n\x07session\x18\x01 \x03(\x0b\x32..allego.SummaSessionsReply.SummaCurrentSession\x1a:\n\x13SummaCurrentSession\x12\x0f\n\x07summaID\x18\x01 \x02(\t\x12\x12\n\ndsourceIDs\x18\x02 \x03(\t\"_\n\x14SummaAnalysisRequest\x12\x0f\n\x07summaID\x18\x01 \x01(\t\x12\x12\n\ndsourceIDs\x18\x02 \x03(\t\x12\x11\n\tpktDurSec\x18\x03 \x01(\x01\x12\x0f\n\x07reqPart\x18\x04 \x01(\t\"\xa1\x02\n\x12SummaAnalysisReply\x12\x0f\n\x07summaID\x18\x01 \x01(\t\x12\x15\n\rallDsourceIDs\x18\x02 \x03(\t\x12!\n\x05state\x18\x03 \x01(\x0b\x32\x12.allego.SummaState\x12\x33\n\x0e\x61llDsourceSpec\x18\x04 \x03(\x0b\x32\x1b.allego.SummaDatasourceSpec\x12-\n\x08\x61ggStats\x18\x05 \x01(\x0b\x32\x1b.allego.SummaAggregateStats\x12@\n\x14reqAvailDsourceStats\x18\x06 \x03(\x0b\x32\".allego.SummaDatasourceAnalysisPkg\x12\x1a\n\x12reqAvailDsourceIDs\x18\x07 \x03(\t\"\xd9\x01\n\nSummaState\x12\x0f\n\x07summaID\x18\x01 \x02(\t\x12\x12\n\nisComplete\x18\x02 \x02(\x08\x12\x14\n\x0c\x66racComplete\x18\x03 \x02(\x01\x12\x0b\n\x03msg\x18\x04 \x02(\t\x12\x19\n\x11\x64srcWalltimeStart\x18\x05 \x02(\t\x12\x17\n\x0f\x64srcWalltimeEnd\x18\x06 \x02(\t\x12\x1d\n\x15\x61nalysisWallTimeStart\x18\x07 \x02(\t\x12\x1a\n\x12\x61nalysisElapsedSec\x18\x08 \x02(\x01\x12\x14\n\x0cpacketDurSec\x18\t \x02(\x01\"]\n\x13SummaAggregateStats\x12%\n\x08metricID\x18\x01 \x03(\x0b\x32\x13.allego.KpiMetricID\x12\x1f\n\x04hist\x18\x02 \x03(\x0b\x32\x11.allego.Histogram\"\xbd\x01\n\x1aSummaDatasourceAnalysisPkg\x12%\n\x08metricID\x18\x01 \x03(\x0b\x32\x13.allego.KpiMetricID\x12\x1f\n\x04hist\x18\x02 \x03(\x0b\x32\x11.allego.Histogram\x12!\n\x04recs\x18\x03 \x03(\x0b\x32\x13.allego.DenseMatrix\x12#\n\x08histRecs\x18\x04 \x03(\x0b\x32\x11.allego.Histogram\x12\x0f\n\x07ntvIdxs\x18\x05 \x03(\x03\"~\n\x13SummaDatasourceSpec\x12\x1f\n\x02sG\x18\x01 \x01(\x0b\x32\x13.allego.SignalGroup\x12\x1d\n\x02tR\x18\x02 \x01(\x0b\x32\x11.allego.TimeRange\x12\'\n\x06niSpec\x18\x03 \x01(\x0b\x32\x17.allego.SpikesSpecReply2\xcb\x33\n\x0bRadiensCore\x12R\n\x0bHealthcheck\x12!.allego.RadiensHealthcheckRequest\x1a\x1e.allego.RadiensHealthcheckSpec\"\x00\x12\x41\n\rClientConnect\x12\x17.allego.RadiensClientId\x1a\x15.allego.StandardReply\"\x00\x12\x44\n\x10\x43lientDisconnect\x12\x17.allego.RadiensClientId\x1a\x15.allego.StandardReply\"\x00\x12\x39\n\x05\x43lose\x12\x17.allego.StandardRequest\x1a\x15.allego.StandardReply\"\x00\x12L\n\x10WorkspaceControl\x12\x1f.allego.WorkspaceControlRequest\x1a\x15.allego.StandardReply\"\x00\x12T\n\x11GetRadiensServers\x12 .allego.GetRadiensServersRequest\x1a\x1b.allego.RadiensServersReply\"\x00\x12H\n\x0cGetWorkspace\x12\x1b.allego.GetWorkspaceRequest\x1a\x19.allego.GetWorkspaceReply\"\x00\x12J\n\x13GetRadixEnvironment\x12\x17.allego.StandardRequest\x1a\x18.allego.RadixEnvironment\"\x00\x12R\n\x17GetOfflineLicenseStatus\x12\x17.allego.StandardRequest\x1a\x1c.allego.OfflineLicenseStatus\"\x00\x12H\n\rListDirectory\x12\x1e.allego.ListDataSourcesRequest\x1a\x15.allego.CpRmMvLsReply\"\x00\x12V\n\x12\x43opyDataSourceFile\x12\'.allego.CopyRemoveDataSourceFileRequest\x1a\x15.allego.CpRmMvLsReply\"\x00\x12X\n\x14RemoveDataSourceFile\x12\'.allego.CopyRemoveDataSourceFileRequest\x1a\x15.allego.CpRmMvLsReply\"\x00\x12P\n\x12MoveDataSourceFile\x12!.allego.MoveDataSourceFileRequest\x1a\x15.allego.CpRmMvLsReply\"\x00\x12N\n\x13SpikesListDirectory\x12\x1e.allego.ListDataSourcesRequest\x1a\x15.allego.CpRmMvLsReply\"\x00\x12R\n\x0eSpikesCopyFile\x12\'.allego.CopyRemoveDataSourceFileRequest\x1a\x15.allego.CpRmMvLsReply\"\x00\x12T\n\x10SpikesRemoveFile\x12\'.allego.CopyRemoveDataSourceFileRequest\x1a\x15.allego.CpRmMvLsReply\"\x00\x12L\n\x0eSpikesMoveFile\x12!.allego.MoveDataSourceFileRequest\x1a\x15.allego.CpRmMvLsReply\"\x00\x12T\n\x16ManageNodeOrganization\x12\x1c.allego.ManageNodeOrgRequest\x1a\x1a.allego.ManageNodeOrgReply\"\x00\x12K\n\rManageNodeLab\x12\x1c.allego.ManageNodeLabRequest\x1a\x1a.allego.ManageNodeLabReply\"\x00\x12N\n\x0eManageNodeUser\x12\x1d.allego.ManageNodeUserRequest\x1a\x1b.allego.ManageNodeUserReply\"\x00\x12\x45\n\x0fManageNodeProbe\x12\x18.allego.ProbeNodeRequest\x1a\x16.allego.ProbeNodeReply\"\x00\x12\x41\n\x0bManageEdges\x12\x18.allego.WorldEdgeRequest\x1a\x16.allego.WorldEdgeReply\"\x00\x12\x42\n\nQueryWorld\x12\x19.allego.QueryWorldRequest\x1a\x17.allego.QueryWorldReply\"\x00\x12V\n\x1bGetWarehouseSensorComponent\x12\x19.allego.SensorCompRequest\x1a\x1a.allego.GetSensorCompReply\"\x00\x12Q\n\x0fListSensorSpecs\x12\x1e.allego.ListSensorSpecsRequest\x1a\x1c.allego.ListSensorSpecsReply\"\x00\x12>\n\tSetSensor\x12\x18.allego.SetSensorRequest\x1a\x15.allego.StandardReply\"\x00\x12T\n\x14SetSensorPositionTcs\x12#.allego.SetSensorPositionTcsRequest\x1a\x15.allego.StandardReply\"\x00\x12R\n\x13SetSitePositionsTcs\x12\".allego.SetSitePositionsTcsRequest\x1a\x15.allego.StandardReply\"\x00\x12N\n\x11UpdateSignalGroup\x12 .allego.UpdateSignalGroupRequest\x1a\x15.allego.StandardReply\"\x00\x12N\n\x11GetSignalGroupIDs\x12\x17.allego.StandardRequest\x1a\x1e.allego.GetSignalGroupIDsReply\"\x00\x12H\n\x0eGetSorterIDMap\x12\x17.allego.StandardRequest\x1a\x1b.allego.GetSorterIDMapReply\"\x00\x12W\n\x11GetSpikeSorterIDs\x12 .allego.GetSpikeSorterIDsRequest\x1a\x1e.allego.GetSpikeSorterIDsReply\"\x00\x12\x45\n\x0eGetSignalGroup\x12\x1c.allego.SignalGroupIDRequest\x1a\x13.allego.SignalGroup\"\x00\x12O\n\x13GetDataSourceParams\x12\x1c.allego.SignalGroupIDRequest\x1a\x18.allego.DataSourceParams\"\x00\x12=\n\nGetSignals\x12\x19.allego.GetSignalsRequest\x1a\x12.allego.RawSignals\"\x00\x12\x41\n\x0cSetDACStream\x12\x18.allego.DACStreamRequest\x1a\x15.allego.StandardReply\"\x00\x12;\n\tSetDACOff\x12\x15.allego.DACOffRequest\x1a\x15.allego.StandardReply\"\x00\x12\x44\n\x0fGetHDSnapshotPy\x12\x1a.allego.HDSnapshotRequest2\x1a\x13.allego.HDSnapshot2\"\x00\x12@\n\rGetHDSnapshot\x12\x19.allego.HDSnapshotRequest\x1a\x12.allego.HDSnapshot\"\x00\x12+\n\x06GetPSD\x12\x12.allego.PSDRequest\x1a\x0b.allego.PSD\"\x00\x12\x42\n\x0bSetDSPGroup\x12\x1a.allego.SetDSPGroupRequest\x1a\x15.allego.StandardReply\"\x00\x12?\n\x0bGetDSPGroup\x12\x1c.allego.SignalGroupIDRequest\x1a\x10.allego.DSPGroup\"\x00\x12J\n\x0eListDataSource\x12\x19.allego.DataSourceRequest\x1a\x1b.allego.DataSourceStatusMap\"\x00\x12K\n\x11ListDataSourceIDs\x12\x19.allego.DataSourceRequest\x1a\x19.allego.DataSourceIDReply\"\x00\x12[\n\x15SetDataSourceFromFile\x12 .allego.DataSourceSetSaveRequest\x1a\x1e.allego.DataSourceSetSaveReply\"\x00\x12I\n\x0f\x43learDataSource\x12\x19.allego.DataSourceRequest\x1a\x19.allego.DataSourceIDReply\"\x00\x12J\n\x10RenameDataSource\x12\x19.allego.DataSourceRequest\x1a\x19.allego.DataSourceIDReply\"\x00\x12P\n\x1aGetDataSourceKpiFileStatus\x12\x19.allego.DataSourceRequest\x1a\x15.allego.KpiFileStatus\"\x00\x12R\n\x1bGetDataSourceKpiFileStatus2\x12\x19.allego.DataSourceRequest\x1a\x16.allego.KpiFileStatus2\"\x00\x12\x46\n\x0eGetKpiMetadata\x12\x19.allego.DataSourceRequest\x1a\x17.allego.KpiFileMetadata\"\x00\x12?\n\x08KpiClear\x12\x1a.allego.KpiStandardRequest\x1a\x15.allego.StandardReply\"\x00\x12\x43\n\x0cKpiCalculate\x12\x1a.allego.KpiStandardRequest\x1a\x15.allego.StandardReply\"\x00\x12\x45\n\x0cGetKpiStatus\x12\x1b.allego.GetKpiStatusRequest\x1a\x16.allego.KpiStatusReply\"\x00\x12H\n\rKpiGetMetrics\x12\x15.allego.KpiMetricsReq\x1a\x1e.allego.KpiBundlePacketMetrics\"\x00\x12\x46\n\x0fSetKpiPacketDur\x12\x1a.allego.SetKpiParamRequest\x1a\x15.allego.StandardReply\"\x00\x12K\n\x0bSetKpiParam\x12#.allego.SpikeSorterSetParamsRequest\x1a\x15.allego.StandardReply\"\x00\x12S\n\x0bGetKpiParam\x12\x19.allego.DataSourceRequest\x1a\'.allego.GetSpikeSorterParamCommandReply\"\x00\x12W\n\x11SpikesSetFromFile\x12 .allego.DataSourceSetSaveRequest\x1a\x1e.allego.DataSourceSetSaveReply\"\x00\x12\x46\n\x0cSpikesGetIDs\x12\x19.allego.DataSourceRequest\x1a\x19.allego.DataSourceIDReply\"\x00\x12H\n\x0eSpikesRenameID\x12\x19.allego.DataSourceRequest\x1a\x19.allego.DataSourceIDReply\"\x00\x12S\n\x17SpikesGetAllSummaryInfo\x12\x19.allego.DataSourceRequest\x1a\x1b.allego.DataSourceStatusMap\"\x00\x12\x45\n\rSpikesGetSpec\x12\x19.allego.DataSourceRequest\x1a\x17.allego.SpikesSpecReply\"\x00\x12\x64\n\x13SpikesGetRasterData\x12\'.allego.SpikeSorterGetRasterDataRequest\x1a\".allego.SpikeSorterRasterDataReply\"\x00\x12[\n\x14SpikesGetSpikesDense\x12\x1e.allego.SpikesGetSpikesRequest\x1a!.allego.SpikesSpikeDataDenseReply\"\x00\x12q\n\x15GetSpikeTrainAnalytic\x12+.allego.SpikesGetSpikeTrainAnalyticsRequest\x1a).allego.SpikesGetSpikeTrainAnalyticsReply\"\x00\x12\x42\n\x0eSpikesExportAs\x12\x17.allego.SpikesExportReq\x1a\x15.allego.StandardReply\"\x00\x12I\n\x0fSpikesDeleteIDs\x12\x19.allego.DataSourceRequest\x1a\x19.allego.DataSourceIDReply\"\x00\x12\x66\n\x16\x42iointerfaceGetNeurons\x12%.allego.BiointerfaceGetNeuronsRequest\x1a#.allego.BiointerfaceGetNeuronsReply\"\x00\x12l\n\"BiointerfaceSeekEndSpikeTimestamps\x12\x31.allego.BiointerfaceSeekEndSpikeTimestampsRequest\x1a\x11.allego.TimeRange\"\x00\x12Q\n\x0f\x42iointerfaceViz\x12\x1e.allego.BiointerfaceVizRequest\x1a\x1c.allego.BiointerfaceVizReply\"\x00\x12[\n\x1b\x42iointerfaceListSensorSpecs\x12\x1c.allego.SignalGroupIDRequest\x1a\x1c.allego.ListSensorSpecsReply\"\x00\x12J\n\x15\x42iointerfaceSetSensor\x12\x18.allego.SetSensorRequest\x1a\x15.allego.StandardReply\"\x00\x12`\n BiointerfaceSetSensorPositionTcs\x12#.allego.SetSensorPositionTcsRequest\x1a\x15.allego.StandardReply\"\x00\x12^\n\x1f\x42iointerfaceSetSitePositionsTcs\x12\".allego.SetSitePositionsTcsRequest\x1a\x15.allego.StandardReply\"\x00\x12H\n\x0c\x46\x65\x61tureStart\x12\x1f.allego.FeatureStartStopRequest\x1a\x15.allego.StandardReply\"\x00\x12G\n\x0b\x46\x65\x61tureStop\x12\x1f.allego.FeatureStartStopRequest\x1a\x15.allego.StandardReply\"\x00\x12=\n\nGetPrivacy\x12\x17.allego.StandardRequest\x1a\x14.allego.PrivacyReply\"\x00\x12@\n\nSetPrivacy\x12\x19.allego.SetPrivacyRequest\x1a\x15.allego.StandardReply\"\x00\x12J\n\x11\x44\x65leteSignalGroup\x12\x1c.allego.SignalGroupIDRequest\x1a\x15.allego.StandardReply\"\x00\x12K\n\x14SpikesGetSignalGroup\x12\x1c.allego.SignalGroupIDRequest\x1a\x13.allego.SignalGroup\"\x00\x12\x33\n\x0bSetProtocol\x12\x10.allego.Protocol\x1a\x10.allego.Protocol\"\x00\x12:\n\x0bGetProtocol\x12\x17.allego.ProtocolRequest\x1a\x10.allego.Protocol\"\x00\x12\x43\n\x0eRenameProtocol\x12\x1d.allego.RenameProtocolRequest\x1a\x10.allego.Protocol\"\x00\x12J\n\x0fGetAllProtocols\x12\x17.allego.StandardRequest\x1a\x1c.allego.GetAllProtocolsReply\"\x00\x12K\n\rApplyProtocol\x12\x17.allego.ProtocolRequest\x1a\x1d.allego.ApplyProtocolProgress\"\x00\x30\x01\x32\xed\t\n\x13RadiensSpikeSorter1\x12R\n\x0bHealthcheck\x12!.allego.RadiensHealthcheckRequest\x1a\x1e.allego.RadiensHealthcheckSpec\"\x00\x12P\n\x12SpikeSorterCommand\x12!.allego.SpikeSorterCommandRequest\x1a\x15.allego.StandardReply\"\x00\x12R\n\x13SpikeSorterSetParam\x12\".allego.SpikeSorterSetParamRequest\x1a\x15.allego.StandardReply\"\x00\x12T\n\x14SpikeSorterSetParams\x12#.allego.SpikeSorterSetParamsRequest\x1a\x15.allego.StandardReply\"\x00\x12\x64\n\x13SpikeSorterGetParam\x12\".allego.SpikeSorterStandardRequest\x1a\'.allego.GetSpikeSorterParamCommandReply\"\x00\x12U\n\x13SpikeSorterGetState\x12\".allego.SpikeSorterStandardRequest\x1a\x18.allego.SpikeSorterState\"\x00\x12\x63\n\x17SpikeSorterGetDashboard\x12#.allego.SpikeSorterDashboardRequest\x1a!.allego.SpikeSorterDashboardReply\"\x00\x12j\n\x1bSpikeSorterGetFeatureParams\x12\".allego.SpikeSorterStandardRequest\x1a%.allego.SpikeSorterFeatureParamsReply\"\x00\x12I\n\x16SpikeSorterWrangleData\x12\x16.allego.WrangleRequest\x1a\x15.allego.StandardReply\"\x00\x12W\n\x11SpikeSorterLaunch\x12 .allego.SpikeSorterLaunchRequest\x1a\x1e.allego.SpikeSorterLaunchReply\"\x00\x12P\n\x11SpikeSorterDelete\x12\".allego.SpikeSorterStandardRequest\x1a\x15.allego.StandardReply\"\x00\x12N\n\rSpikesGetSpec\x12\".allego.SpikeSorterStandardRequest\x1a\x17.allego.SpikesSpecReply\"\x00\x12\x66\n\x16SpikeSorterIsAnyActive\x12%.allego.SpikeSorterIsAnyActiveRequest\x1a#.allego.SpikeSorterIsAnyActiveReply\"\x00\x12J\n\x17SpikeSorterImportMearec\x12\x14.allego.RecgenImport\x1a\x17.allego.FileDescriptors\"\x00\x32\xe2\x03\n\nDashboards\x12R\n\x0bHealthcheck\x12!.allego.RadiensHealthcheckRequest\x1a\x1e.allego.RadiensHealthcheckSpec\"\x00\x12L\n\x10\x43ommandDashboard\x12\x1f.allego.DashboardCommandRequest\x1a\x15.allego.StandardReply\"\x00\x12S\n\x15LaunchSessionAnalysis\x12\x1c.allego.SummaAnalysisRequest\x1a\x1a.allego.SummaAnalysisReply\"\x00\x12P\n\x12GetSessionAnalysis\x12\x1c.allego.SummaAnalysisRequest\x1a\x1a.allego.SummaAnalysisReply\"\x00\x12\x45\n\x0cListSessions\x12\x17.allego.StandardRequest\x1a\x1a.allego.SummaSessionsReply\"\x00\x12\x44\n\x10\x43learAllSessions\x12\x17.allego.StandardRequest\x1a\x15.allego.StandardReply\"\x00\x32\xef\x05\n\x0bRadiensDev1\x12R\n\x0bHealthcheck\x12!.allego.RadiensHealthcheckRequest\x1a\x1e.allego.RadiensHealthcheckSpec\"\x00\x12S\n\x1aSpkSortTrainerMakeNewModel\x12\x1c.allego.SpkSortTrainerNewReq\x1a\x15.allego.StandardReply\"\x00\x12]\n\x1eSpkSortTrainerLearnHyperParams\x12\".allego.SpkSortLearnHyperParamsReq\x1a\x15.allego.StandardReply\"\x00\x12W\n\x1aSpkSortTrainerProfileModel\x12 .allego.SpkSortTrainerProfileReq\x1a\x15.allego.StandardReply\"\x00\x12W\n#SpkSortTrainerMakeDefaultProdModels\x12\x17.allego.StandardRequest\x1a\x15.allego.StandardReply\"\x00\x12\x64\n$SpkSortTrainerListLocalNetworkModels\x12\x17.allego.StandardRequest\x1a!.allego.SpkSortTrainerStatusReply\"\x00\x12g\n&SpkSortTrainerDeleteLocalNetworkModels\x12$.allego.SpkSortTrainerDeleteModelReq\x1a\x15.allego.StandardReply\"\x00\x12W\n\x17SpkSortTrainerGetStatus\x12\x17.allego.StandardRequest\x1a!.allego.SpkSortTrainerStatusReply\"\x00\x42\x15Z\x13internal/radix/grpc')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'radiensserver_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'Z\023internal/radix/grpc'
  _RADIENSHEALTHCHECKREQUEST._serialized_start=121
  _RADIENSHEALTHCHECKREQUEST._serialized_end=196
  _RADIENSHEALTHCHECKSPEC._serialized_start=198
  _RADIENSHEALTHCHECKSPEC._serialized_end=291
  _RADIENSCLIENTID._serialized_start=293
  _RADIENSCLIENTID._serialized_end=323
  _SUMMASESSIONSREPLY._serialized_start=326
  _SUMMASESSIONSREPLY._serialized_end=471
  _SUMMASESSIONSREPLY_SUMMACURRENTSESSION._serialized_start=413
  _SUMMASESSIONSREPLY_SUMMACURRENTSESSION._serialized_end=471
  _SUMMAANALYSISREQUEST._serialized_start=473
  _SUMMAANALYSISREQUEST._serialized_end=568
  _SUMMAANALYSISREPLY._serialized_start=571
  _SUMMAANALYSISREPLY._serialized_end=860
  _SUMMASTATE._serialized_start=863
  _SUMMASTATE._serialized_end=1080
  _SUMMAAGGREGATESTATS._serialized_start=1082
  _SUMMAAGGREGATESTATS._serialized_end=1175
  _SUMMADATASOURCEANALYSISPKG._serialized_start=1178
  _SUMMADATASOURCEANALYSISPKG._serialized_end=1367
  _SUMMADATASOURCESPEC._serialized_start=1369
  _SUMMADATASOURCESPEC._serialized_end=1495
  _RADIENSCORE._serialized_start=1498
  _RADIENSCORE._serialized_end=8101
  _RADIENSSPIKESORTER1._serialized_start=8104
  _RADIENSSPIKESORTER1._serialized_end=9365
  _DASHBOARDS._serialized_start=9368
  _DASHBOARDS._serialized_end=9850
  _RADIENSDEV1._serialized_start=9853
  _RADIENSDEV1._serialized_end=10604
# @@protoc_insertion_point(module_scope)
