from pprint import pprint
import unittest
from pathlib import Path

import radiens.utils.config as cfg


class base_utest(unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        pass

    def setUp(self):
        pass
    
    def test_discover_server_unix(self):
        pass

    

if __name__ == "__main__":
    unittest.main()
